@@ -20,11 +20,14 @@
 #include <string.h>
 #include <errno.h>
 #include <unistd.h>
+#include <sys/ioctl.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/wait.h>
 #include <sys/time.h>
 #include <sys/poll.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 #include <limits.h>
 #include <getopt.h>
 #include <signal.h>
@@ -46,6 +49,7 @@ static unsigned int fifo = 0;
 static unsigned int process_mode = PROCESS_MODE;
 
 static int use_pipes = 0;
+static int use_inet = 0;
 
 struct sender_context {
 	unsigned int num_fds;
@@ -99,6 +103,7 @@ static void print_usage_exit(int error)
 	       "-h       --help            print this message\n"
 	       "-l       --loops=LOOPS     how many message should be send\n"
 	       "-p       --pipe            send data via a pipe\n"
+	       "-i       --inet            send data via a inet tcp connection\n"
 	       "-s       --datasize=SIZE   message size\n"
 	       "-T       --threads         use POSIX threads\n"
 	       "-P       --process         use fork (default)\n"
@@ -106,11 +111,54 @@ static void print_usage_exit(int error)
 	exit(error);
 }
 
+static int inet_socketpair(int fds[2])
+{
+	int s1, s2;
+	struct sockaddr_in sin;
+	unsigned long ul = 1;
+
+	if ((s1 = socket(AF_INET, SOCK_STREAM, 0)) < 0)
+		barf("socket");
+	if ((s2 = socket(AF_INET, SOCK_STREAM, 0)) < 0)
+		barf("socket");
+
+	socklen_t len = sizeof(sin);
+	bzero(&sin, len);
+	sin.sin_family = AF_INET;
+	sin.sin_port = 0;
+	sin.sin_addr.s_addr = inet_addr("127.0.0.1");
+
+	if (bind(s1, &sin, len) < 0)
+		barf("bind");
+	if (getsockname(s1, &sin, &len) < 0)
+		barf("getsockname");
+	if (listen(s1, 10) < 0)
+		barf("listen");
+	if (ioctl(s2, FIONBIO, &ul) < 0)
+		barf("ioctl");
+	if (ioctl(s1, FIONBIO, &ul) < 0)
+		barf("ioctl");
+	if (connect(s2, &sin, len) < 0)
+		barf("connect");
+	if ((fds[0] = accept(s1, &sin, &len)) < 0)
+		barf("accept");
+	ul = 0;
+	if (ioctl(s2, FIONBIO, &ul) < 0)
+		barf("ioctl");
+	fds[1] = s2;
+	close(s1);
+
+	return 0;
+}
+
 static void fdpair(int fds[2])
 {
 	if (use_pipes) {
 		if (pipe(fds) == 0)
 			return;
+	} else if (use_inet) {
+		if (inet_socketpair(fds) == 0)
+			return;
 	} else {
 		if (socketpair(AF_UNIX, SOCK_STREAM, 0, fds) == 0)
 			return;
@@ -152,7 +200,8 @@ static void *sender(struct sender_context *ctx)
 	/* Now pump to every receiver. */
 	for (i = 0; i < loops; i++) {
 		for (j = 0; j < ctx->num_fds; j++) {
-			int ret, done = 0;
+			int ret;
+			size_t done = 0;
 
 again:
 			ret = write(ctx->out_fds[j], data + done, sizeof(data)-done);
@@ -183,7 +232,8 @@ static void *receiver(struct receiver_context* ctx)
 	/* Receive them all */
 	for (i = 0; i < ctx->num_packets; i++) {
 		char data[datasize];
-		int ret, done = 0;
+		int ret;
+		size_t done = 0;
 
 again:
 		ret = read(ctx->in_fds[0], data + done, datasize - done);
@@ -241,7 +291,7 @@ static int create_worker(childinfo_t *child, void *ctx, void *(*func)(void *))
 
 void signal_workers(childinfo_t *children, unsigned int num_children)
 {
-	int i;
+	unsigned int i;
 	printf("signaling %d worker threads to terminate\n", num_children);
 	for (i=0; i < num_children; i++) {
 		kill(children[i].pid, SIGTERM);
@@ -364,13 +414,14 @@ static void process_options(int argc, char *argv[])
 			{"help",	no_argument,		NULL, 'h'},
 			{"loops",	required_argument,	NULL, 'l'},
 			{"pipe",	no_argument,		NULL, 'p'},
+			{"inet",	no_argument,		NULL, 'i'},
 			{"datasize",	required_argument,	NULL, 's'},
 			{"threads",	no_argument,		NULL, 'T'},
 			{"processes",	no_argument,		NULL, 'P'},
 			{NULL, 0, NULL, 0}
 		};
 
-		int c = getopt_long(argc, argv, "f:Fg:hl:ps:TP",
+		int c = getopt_long(argc, argv, "f:Fg:hl:pis:TP",
 				    longopts, &optind);
 		if (c == -1) {
 			break;
@@ -393,6 +444,7 @@ static void process_options(int argc, char *argv[])
 			break;
 		case 'h':
 			print_usage_exit(0);
+			break;
 		case 'l':
 			if (!(argv[optind] && (loops = atoi(optarg)) > 0)) {
 				fprintf(stderr, "%s: --loops|-l requires an integer > 0\n", argv[0]);
@@ -402,6 +454,9 @@ static void process_options(int argc, char *argv[])
 		case 'p':
 			use_pipes = 1;
 			break;
+		case 'i':
+			use_inet = 1;
+			break;
 		case 's':
 			if (!(argv[optind] && (datasize = atoi(optarg)) > 0)) {
 				fprintf(stderr, "%s: --datasize|-s requires an integer > 0\n", argv[0]);
@@ -418,6 +473,11 @@ static void process_options(int argc, char *argv[])
 			print_usage_exit(1);
 		}
 	}
+
+	if (use_pipes && use_inet) {
+		fprintf(stderr, "%s: --pipe|-p and --inet|-i cannot be used together\n", argv[0]);
+		print_usage_exit(1);
+	}
 }
 
 void sigcatcher(int sig) {
@@ -460,7 +520,7 @@ int main(int argc, char *argv[])
 	if (setjmp(jmpbuf) == 0) {
 		total_children = 0;
 		for (i = 0; i < num_groups; i++) {
-			int c = group(child_tab, total_children, num_fds, readyfds[1], wakefds[0]);
+			unsigned int c = group(child_tab, total_children, num_fds, readyfds[1], wakefds[0]);
 			if( c != (num_fds*2) ) {
 				fprintf(stderr, "%i children started.  Expected %i\n", c, num_fds*2);
 				reap_workers(child_tab, total_children + c, 1);
