@@ -1,4 +1,5 @@
 #!/usr/bin/python3
+""" Module to detect smis """
 
 # SPDX-License-Identifier: GPL-2.0-only
 
@@ -6,23 +7,31 @@
 # (C) 2015,2016 Clark Williams <williams@redhat.com>
 # (C) 2009 Clark Williams <williams@redhat.com>
 
-import sys
-import os
-import time
-import subprocess
+import abc
 import errno
+import os
 import os.path
+import subprocess
+import sys
+import time
 
 version = "0.8"
 debugging = False
 quiet = False
 watch = False
 
-def debug(str):
-    if debugging: print(str)
 
-def info(str):
-    if not quiet: print(str)
+def debug(dstr):
+    """ print debugging string if debugging is on """
+    if debugging:
+        print(dstr)
+
+
+def info(istr):
+    " print info string if quiet is not on """
+    if not quiet:
+        print(istr)
+
 
 #
 # Class used to manage mounting and umounting the debugfs
@@ -31,26 +40,26 @@ def info(str):
 # discovers that debugfs is already mounted, it will leave
 # it mounted.
 #
-class DebugFS(object):
-    '''class to manage mounting/umounting the debugfs'''
+class DebugFS:
+    """ class to manage mounting/umounting the debugfs """
     def __init__(self):
         self.premounted = False
         self.mounted = False
         self.mountpoint = ''
-        f = open('/proc/mounts')
-        for l in f:
-            field = l.split()
-            if field[2] == "debugfs":
-                self.premounted = True
-                self.mountpoint = field[1]
-                break
-        f.close()
+        with open('/proc/mounts') as f:
+            for l in f:
+                field = l.split()
+                if field[2] == "debugfs":
+                    self.premounted = True
+                    self.mountpoint = field[1]
+                    break
 
     def mount(self, path='/sys/kernel/debug'):
+        """ mount debugfs unless already mounted """
         if self.premounted or self.mounted:
             debug("not mounting debugfs")
             return True
-        debug("mounting debugfs at %s" % path)
+        debug(f"mounting debugfs at {path}")
         self.mountpoint = path
         cmd = ['/bin/mount', '-t', 'debugfs', 'none', path]
         self.mounted = (subprocess.call(cmd) == 0)
@@ -59,33 +68,29 @@ class DebugFS(object):
         return self.mounted
 
     def umount(self):
+        """ unmount debugfs if not premounted """
         if self.premounted or not self.mounted:
             debug("not umounting debugfs")
             return True
         debug("umounting debugfs")
         cmd = ['/bin/umount', self.mountpoint]
-        self.mounted = not (subprocess.call(cmd) == 0)
+        self.mounted = subprocess.call(cmd) != 0
         if self.mounted:
             raise RuntimeError("Failed to umount debugfs")
         return not self.mounted
 
     def getval(self, item, nonblocking=False):
+        """ get value of mountpoint/item """
         path = os.path.join(self.mountpoint, item)
-        if nonblocking == False:
-            f = open(path)
-            val = f.readline()
-            f.close()
+        if nonblocking is False:
+            with open(path) as f:
+                val = f.readline()
         else:
-            f = os.fdopen(os.open(path, os.O_RDONLY|os.O_NONBLOCK), "r")
+            f = os.fdopen(os.open(path, os.O_RDONLY | os.O_NONBLOCK), "r")
             try:
                 val = f.readline()
             except OSError as e:
-                print("errno: %s" % e)
-                if e.errno == errno.EAGAIN:
-                    val = None
-                else:
-                    raise
-            except IOError as e:
+                print(f"errno: {e}")
                 if e.errno == errno.EAGAIN:
                     val = None
                 else:
@@ -94,98 +99,29 @@ class DebugFS(object):
         return val
 
     def putval(self, item, value):
+        """ write value to mountpoint/item """
         path = os.path.join(self.mountpoint, item)
-        f = open(path, "w")
-        f.write(str(value))
-        f.flush()
-        f.close()
+        with open(path, "w") as f:
+            f.write(str(value))
+            f.flush()
 
     def getpath(self, item):
+        """ get mountpoint/item """
         return os.path.join(self.mountpoint, item)
 
-# Exception class for when tracer is not available
+
 class DetectorNotAvailable(Exception):
+    """ Exception class for when tracer is not available """
+
     def __init__(self, name, msg):
         self.args = (name, msg)
         self.name = name
         self.msg = msg
 
-#
-# Class used to manage loading and unloading of the
-# hwlat kernel module. Like the debugfs class
-# above, if the module is already loaded, this class will
-# leave it alone when cleaning up.
-#
-class Kmod(object):
-    ''' class to manage loading and unloading of kernel modules'''
 
-    names = ("hwlat_detector", "smi_detector")
+class Detector:
+    """ base class for detector modules """
 
-    def __check_builtin(self):
-        for l in open(os.path.join('/lib/modules', os.uname()[2], 'modules.builtin'), "r"):
-            if self.name in l:
-                debug("found %s as builtin" % self.namename)
-                return True
-        return False
-
-    def __find_module(self):
-        debug("looking for module %s" % self.name)
-        path = os.path.join("/lib/modules",
-                            os.uname()[2],
-                            "kernel/drivers/misc")
-        debug("module path: %s" % path)
-        mpath = os.path.join(path, self.name) + ".ko"
-        debug("checking %s" % mpath)
-        if os.path.exists(mpath):
-            return True
-        return False
-
-    def __init__(self, name):
-        if name not in Kmod.names:
-            raise RuntimeError("unsupported module name: %s" % name)
-        if name == "smi_detector":
-            raise RuntimeError("smi_detector module no longer supported!")
-        self.name = name
-        self.preloaded = False
-        self.builtin = False
-
-        # check for builtin
-        if self.__check_builtin():
-            self.builtin = True
-            return
-
-        # now look for already loaded module
-        for l in open('/proc/modules'):
-            field = l.split()
-            if self.name in field[0]:
-                self.preloaded = True
-                debug("using already loaded %s" % self.name)
-                return
-        if not self.__find_module():
-            raise DetectorNotAvailable(name, "module %s does not exist!" % self.name)
-
-    def load(self):
-        if self.builtin:
-            debug("not loading %s (builtin)" % self.name)
-            return True
-        if self.preloaded:
-            debug("not loading %s (already loaded)" % self.name)
-            return True
-        cmd = ['/sbin/modprobe', self.name]
-        return (subprocess.call(cmd) == 0)
-
-    def unload(self):
-        if self.preloaded or self.builtin:
-            debug("Not unloading %s" % self.name)
-            return True
-        cmd = ['/sbin/modprobe', '-r', self.name]
-        return (subprocess.call(cmd) == 0)
-
-#
-# base class for detection modules
-#
-class Detector(object):
-    '''base class for detector modules'''
     def __init__(self):
         self.type = "unknown"
         if os.getuid() != 0:
@@ -194,12 +130,13 @@ class Detector(object):
         if not self.debugfs.mount():
             raise RuntimeError("failed to mount debugfs")
         self.samples = []
-        self.testduration = 30 # ten seconds
+        self.testduration = 30  # in seconds
         self.have_msr = False
         self.initsmi = []
         if os.path.exists('/usr/sbin/rdmsr'):
             self.have_msr = True
             self.initsmi = self.getsmicounts()
+        self.dma_latency_handle = None
 
     def getsmicounts(self):
         counts = []
@@ -210,58 +147,66 @@ class Detector(object):
         return counts
 
     # methods for preventing/enabling c-state transitions
-    # openinging /dev/cpu_dma_latency and writeing a 32-bit zero to that file will prevent
-    # c-state transitions while the file descriptor is open.
+    #
+    # opening /dev/cpu_dma_latency and writing a 32-bit zero to that file will
+    # prevent c-state transitions while the file descriptor is open.
+    #
     # use c_states_off() to disable c-state transitions
     # use c_states_on() to close the file descriptor and re-enable c-states
     #
     def c_states_off(self):
-        self.dma_latency_handle = os.open("/dev/cpu_dma_latency", os.O_WRONLY)
-        os.write(self.dma_latency_handle, b'\x00\x00\x00\x00')
-        debug("c-states disabled")
+        """ disable c-state transitions """
+        if os.path.exists("/dev/cpu_dma_latency"):
+            self.dma_latency_handle = os.open("/dev/cpu_dma_latency", os.O_WRONLY)
+            os.write(self.dma_latency_handle, b'\x00\x00\x00\x00')
+            debug("c-states disabled")
 
     def c_states_on(self):
+        """ close the file descriptor and re-enable c-states """
         if self.dma_latency_handle:
             os.close(self.dma_latency_handle)
-        debug("c-states enabled")
+            debug("c-states enabled")
 
+    @abc.abstractmethod
     def cleanup(self):
-        raise RuntimeError("must override base method 'cleanup'!")
+        ''' abstract cleanup method, must override '''
 
+    @abc.abstractmethod
     def get(self, field):
-        '''get the value of a debugfs field'''
-        raise RuntimeError("must override base method 'get'!")
+        ''' get the value of a debugfs field '''
 
+    @abc.abstractmethod
     def set(self, field, val):
-        '''set a value in a debugfs field'''
-        raise RuntimeError("must override base method 'set'!")
+        ''' set a value in a debugfs field '''
 
-    def save(self, reportfile=None):
-        '''save sample data to reportfile'''
-        raise RuntimeError("must override base method 'save'!")
+    @abc.abstractmethod
+    def save(self, output=None):
+        ''' save sample data to output '''
 
+    @abc.abstractmethod
     def display(self):
-        '''output the sample data as a string'''
-        raise RuntimeError("must override base method 'display'!")
+        ''' output the sample data as a string '''
 
     def start(self):
+        """ enable the detector """
         count = 0
         threshold = int(self.get("threshold"))
         self.c_states_off()
-        debug("enabling detector module (threshold: %d)" % threshold)
+        debug(f"enabling detector module (threshold: {threshold})")
         self.set("enable", 1)
         while self.get("enable") == 0:
             debug("still disabled, retrying in a bit")
             count += 1
             time.sleep(0.1)
-            debug("retrying enable of detector module (%d)" % count)
+            debug(f"retrying enable of detector module ({count})")
             self.set("enable", 1)
         if self.get("threshold") != threshold:
             debug("start: threshold reset by start, fixing")
             self.set("threshold", threshold)
-        debug("detector module enabled (threshold: %d)" % int(self.get("threshold")))
+        debug(f"detector module enabled (threshold: {int(self.get('threshold'))})")
 
     def stop(self):
+        """ disable the detector """
         count = 0
         debug("disabling detector module")
         self.set("enable", 0)
@@ -269,47 +214,49 @@ class Detector(object):
             debug("still enabled, retrying in a bit")
             count += 1
             time.sleep(0.1)
-            debug("retrying disable of detector module(%d)" % count)
+            debug(f"retrying disable of detector module({count})")
             self.set("enable", 0)
         self.c_states_on()
         debug("detector module disabled")
 
+    @abc.abstractmethod
     def detect(self):
-        '''get detector output'''
-        raise RuntimeError("must override base method 'detect'!")
-#
-# class to handle running the hwlat tracer module of ftrace
-#
+        ''' get detector output '''
+
+
 class Tracer(Detector):
-    '''class to wrap access to ftrace hwlat tracer'''
+    """ Class to handle running the hwlat tracer module of ftrace """
+
     __field_translation = {
         'width'     : "hwlat_detector/width",
         'window'    : "hwlat_detector/window",
         'enable'    : "tracing_on",
         'threshold' : "tracing_thresh",
+        'cpumask'   : "tracing_cpumask",
     }
 
-    class Sample(object):
+    class Sample:
         'private class for tracer sample data'
-        __slots__ = 'timestamp', 'inner', 'outer',
+        __slots__ = 'timestamp', 'inner', 'outer'
+
         def __init__(self, line):
             fields = line.split()
-            i,o = fields[6].split('/')
+            i, o = fields[6].split('/')
             ts = fields[7][3:]
             self.timestamp = str(ts)
             self.inner = int(i)
             self.outer = int(o)
 
         def __str__(self):
-            return "ts: %s, inner:%d, outer:%d" % (self.timestamp, self.inner, self.outer)
+            return f"ts: {self.timestamp}, inner:{self.inner}, outer:{self.outer}"
 
         def display(self):
+            """ convert object to string and print """
             print(str(self))
 
         def largest(self):
-            if self.inner > self.outer:
-                return self.inner
-            return self.outer
+            """ return largest value of inner or outer """
+            return max(self.inner, self.outer)
 
     def translate(self, field):
         path = self.debugfs.getpath('tracing')
@@ -318,7 +265,7 @@ class Tracer(Detector):
         return os.path.join(path, Tracer.__field_translation[field])
 
     def __init__(self):
-        super(Tracer, self).__init__()
+        super().__init__()
         path = self.debugfs.getpath('tracing/hwlat_detector')
         if not os.path.exists(path):
             raise DetectorNotAvailable("hwlat", "hwlat tracer not available")
@@ -334,7 +281,7 @@ class Tracer(Detector):
     def get(self, field):
         if field == "count":
             return len(self.samples)
-        elif field == "max":
+        if field == "max":
             max = 0
             for values in self.samples:
                 s = int(values.largest())
@@ -354,7 +301,8 @@ class Tracer(Detector):
                 val = self.get_sample()
                 while val:
                     self.samples.append(val)
-                    if watch: val.display()
+                    if watch:
+                        val.display()
                     val = self.get_sample()
                 time.sleep(0.1)
         except KeyboardInterrupt as e:
@@ -370,11 +318,12 @@ class Tracer(Detector):
         return val
 
     def save(self, output=None):
+        """ save samples """
         if output:
-            f = open(output, "w")
-            for s in self.samples:
-                f.write("%s\n" % str(s))
-            print("report saved to %s (%d samples)" % (output, len(self.samples)))
+            with open(output, "w") as f:
+                for s in self.samples:
+                    f.write(f"{s}\n")
+                print(f"report saved to {output} ({len(self.samples)} samples)")
 
     def display(self):
         for s in self.samples:
@@ -387,116 +336,52 @@ class Tracer(Detector):
             raise RuntimeError("Failed to unmount debugfs")
 
 
-#
-# Class to simplify running the hwlat kernel module
-#
-class Hwlat(Detector):
-    '''class to wrap access to hwlat debugfs files'''
-    def __init__(self):
-        super(Hwlat, self).__init__()
-        self.kmod = Kmod("hwlat_detector")
-        self.type = "kmodule"
-        self.kmod.load()
-
-    def get(self, field):
-        return int(self.debugfs.getval(os.path.join("hwlat_detector", field)))
-
-    def set(self, field, val):
-        if field == "enable" and val:
-            val = 1
-        self.debugfs.putval(os.path.join("hwlat_detector", field), str(val))
-
-    def get_sample(self):
-        return self.debugfs.getval("hwlat_detector/sample", nonblocking=True)
-
-    def detect(self):
-        self.samples = []
-        testend = time.time() + self.testduration
-        pollcnt = 0
-        self.start()
-        try:
-            while time.time() < testend:
-                pollcnt += 1
-                val = self.get_sample()
-                while val:
-                    val = val.strip()
-                    self.samples.append(val)
-                    if watch: print(val)
-                    val = self.get_sample()
-                time.sleep(0.1)
-        except KeyboardInterrupt as e:
-            print("interrupted")
-        self.stop()
-        return self.samples
-
-    def display(self):
-        for s in self.samples:
-            print(s)
-
-    def save(self, output=None):
-        if output:
-            f = open(output, "w")
-            for s in self.samples:
-                f.write("%s\n" % str(s))
-            print("report saved to %s (%d samples)" % (output, len(self.samples)))
-
-    def cleanup(self):
-        if not self.kmod.unload():
-            raise RuntimeError("Failed to unload %s" % self.name)
-        if not self.debugfs.umount():
-            raise RuntimeError("Failed to unmount debugfs")
-
-def seconds(str):
+def seconds(sval):
     "convert input string to value in seconds"
-    if str.isdigit():
-        return int(str)
-    elif str[-2].isalpha():
-        raise RuntimeError("illegal suffix for seconds: '%s'" % str[-2:-1])
-    elif str[-1:] == 's':
-        return int(str[0:-1])
-    elif str[-1:] == 'm':
-        return int(str[0:-1]) * 60
-    elif str[-1:] == 'h':
-        return int(str[0:-1]) * 3600
-    elif str[-1:] == 'd':
-        return int(str[0:-1]) * 86400
-    elif str[-1:] == 'w':
-        return int(str[0:-1]) * 86400 * 7
-    else:
-        raise RuntimeError("invalid input for seconds: '%s'" % str)
+    if sval.isdigit():
+        return int(sval)
+    if sval[-2].isalpha():
+        raise RuntimeError(f"illegal suffix for seconds: '{sval[-2:-1]}'")
+    if sval[-1:] == 's':
+        return int(sval[0:-1])
+    if sval[-1:] == 'm':
+        return int(sval[0:-1]) * 60
+    if sval[-1:] == 'h':
+        return int(sval[0:-1]) * 3600
+    if sval[-1:] == 'd':
+        return int(sval[0:-1]) * 86400
+    if sval[-1:] == 'w':
+        return int(sval[0:-1]) * 86400 * 7
+    raise RuntimeError(f"invalid input for seconds: '{sval}'")
 
-def milliseconds(str):
+
+def milliseconds(sval):
     "convert input string to millsecond value"
-    if str.isdigit():
-        return int(str)
-    elif str[-2:] == 'ms':
-        return int(str[0:-2])
-    elif str[-1] == 's':
-        return int(str[0:-2]) * 1000
-    elif str[-1] == 'm':
-        return int(str[0:-1]) * 1000 * 60
-    elif str[-1] == 'h':
-        return int(str[0:-1]) * 1000 * 60 * 60
-    else:
-        raise RuntimeError("invalid input for milliseconds: %s" % str)
+    if sval.isdigit():
+        return int(sval)
+    if sval[-2:] == 'ms':
+        return int(sval[0:-2])
+    if sval[-1] == 's':
+        return int(sval[0:-2]) * 1000
+    if sval[-1] == 'm':
+        return int(sval[0:-1]) * 1000 * 60
+    if sval[-1] == 'h':
+        return int(sval[0:-1]) * 1000 * 60 * 60
+    raise RuntimeError(f"invalid input for milliseconds: {sval}")
 
 
-def microseconds(str):
+def microseconds(sval):
     "convert input string to microsecond value"
-    if str.isdigit():
-        return int(str)
-    elif str[-2:] == 'ms':
-        return (int(str[0:-2]) * 1000)
-    elif str[-2:] == 'us':
-        return int(str[0:-2])
-    elif str[-1:] == 's':
-        return (int(str[0:-1]) * 1000 * 1000)
-    else:
-        raise RuntimeError("invalid input for microseconds: '%s'" % str)
+    if sval.isdigit():
+        return int(sval)
+    if sval[-2:] == 'ms':
+        return int(sval[0:-2]) * 1000
+    if sval[-2:] == 'us':
+        return int(sval[0:-2])
+    if sval[-1:] == 's':
+        return int(sval[0:-1]) * 1000 * 1000
+    raise RuntimeError(f"invalid input for microseconds: '{sval}'")
 
-#
-# main starts here
-#
 
 if __name__ == '__main__':
     from argparse import ArgumentParser
@@ -526,6 +411,10 @@ if __name__ == '__main__':
                         dest="report",
                         help="filename for sample data")
 
+    parser.add_argument("--cpu-list", default=None,
+                        dest="cpulist",
+                        help="the CPUs for hwlat thread to move across")
+
     parser.add_argument("--debug", action="store_true", default=False,
                         dest="debug",
                         help="turn on debugging prints")
@@ -555,51 +444,66 @@ if __name__ == '__main__':
     if args.threshold:
         t = microseconds(args.threshold)
         detect.set("threshold", t)
-        debug("threshold set to %dus" % t)
+        debug(f"threshold set to {t}us")
 
     if args.hardlimit:
         hardlimit = microseconds(args.hardlimit)
     else:
         hardlimit = int(detect.get("threshold"))
-    debug("hardlimit set to %dus" % hardlimit)
+    debug(f"hardlimit set to {hardlimit}us")
 
     if args.window:
         w = microseconds(args.window)
+        width = w//2
         if w < int(detect.get("width")):
-            debug("shrinking width to %d for new window of %d" % (w/2, w))
-            detect.set("width", w/2)
-        debug("window parameter = %d" % w)
+            debug(f"shrinking width to {width} for new window of {w}")
+            detect.set("width", width)
+        debug(f"window parameter = {w}")
         detect.set("window", w)
-        debug("window for sampling set to %dus" % w)
+        debug(f"window for sampling set to {w}us")
 
     if args.width:
         w = microseconds(args.width)
         if w > int(detect.get("window")):
-            debug("widening window to %d for new width of %d" % (w*2, w))
+            debug(f"widening window to {w*2} for new width of {w}")
             detect.set("window", w*2)
-        debug("width parameter = %d" % w)
+        debug(f"width parameter = {w}")
         detect.set("width", w)
-        debug("sample width set to %dus" % w)
+        debug(f"sample width set to {w}us")
 
     if args.duration:
         detect.testduration = seconds(args.duration)
     else:
-        detect.testduration = 120 # 2 minutes
-    debug("test duration is %ds" % detect.testduration)
+        detect.testduration = 120  # 2 minutes
+    debug(f"test duration is {detect.testduration}s")
 
     if args.watch:
         watch = True
 
     reportfile = args.report
 
-    info("hwlatdetect:  test duration %d seconds" % detect.testduration)
-    info("   detector: %s" % detect.type)
+    if args.cpulist:
+        cpumask = 0
+        for c in args.cpulist.split(','):
+            l, r = 0, 0
+            if '-' in c:
+                l, r = map(int, c.split('-'))
+            else:
+                l, r = map(int, [c, c])
+            for i in range(l, r + 1):
+                cpumask |= (1 << i)
+        debug(f"set tracing_cpumask to {cpumask:x}")
+        detect.set("cpumask", f"{cpumask:x}")
+
+    info(f"hwlatdetect:  test duration {detect.testduration} seconds")
+    info(f"   detector: {detect.type}")
     info("   parameters:")
-    info("        Latency threshold: %dus" % int(detect.get("threshold")))
-    info("        Sample window:     %dus" % int(detect.get("window")))
-    info("        Sample width:      %dus" % int(detect.get("width")))
-    info("     Non-sampling period:  %dus" % (int(detect.get("window")) - int(detect.get("width"))))
-    info("        Output File:       %s" % reportfile)
+    info(f"        CPU list:          {args.cpulist}")
+    info(f"        Latency threshold: {int(detect.get('threshold'))}us")
+    info(f"        Sample window:      {int(detect.get('window'))}us")
+    info(f"        Sample width:      {int(detect.get('width'))}us")
+    info(f"     Non-sampling period:  {(int(detect.get('window')) - int(detect.get('width')))}us")
+    info(f"        Output File:       {reportfile}")
     info("\nStarting test")
 
     detect.detect()
@@ -610,22 +514,22 @@ if __name__ == '__main__':
     if max_latency == 0:
         info("Max Latency: Below threshold")
     else:
-        info("Max Latency: %dus" % max_latency)
+        info(f"Max Latency: {int(max_latency)}us")
 
-    info("Samples recorded: %d" % len(detect.samples))
+    info(f"Samples recorded: {len(detect.samples)}")
 
     exceeding = detect.get("count")
-    info("Samples exceeding threshold: %d" % exceeding)
+    info(f"Samples exceeding threshold: {exceeding}")
 
     if detect.have_msr:
         finishsmi = detect.getsmicounts()
         total_smis = 0
-        for i,count in enumerate(finishsmi):
+        for i, count in enumerate(finishsmi):
             if count > detect.initsmi[i]:
                 smis = count - detect.initsmi[i]
                 total_smis += smis
-                print("%d SMIs occured on cpu %d" % (smis, i))
-        info("SMIs during run: %d" % total_smis)
+                print(f"{smis} SMIs occured on cpu {i}")
+        info(f"SMIs during run: {total_smis}")
 
     maxlatency = int(detect.get("max"))
 
